@@ -6,15 +6,15 @@ use rand::{rngs::SmallRng, Rng, SeedableRng};
 
 use criterion::{black_box, criterion_group, criterion_main, Criterion};
 
-pub fn csprng_setup<E: Engine, const DEGREE_LIMIT: usize>() -> KZGParams<E, DEGREE_LIMIT> {
+pub fn csprng_setup<E: Engine, const MAX_COEFFS: usize>() -> KZGParams<E> {
     let s: E::Fr = rand::random::<u64>().into();
-    setup(s)
+    setup(s, MAX_COEFFS)
 }
 
 fn bench_commit<E: Engine, const NUM_COEFFS: usize>(c: &mut Criterion) {
     let params = csprng_setup::<E, NUM_COEFFS>();
     let mut rng = SmallRng::from_seed([42; 32]);
-    let mut coeffs = [E::Fr::zero(); NUM_COEFFS];
+    let mut coeffs = vec![E::Fr::zero(); NUM_COEFFS];
     for i in 0..NUM_COEFFS {
         coeffs[i] = rng.gen::<u64>().into();
     }
